

io.stdout:setvbuf('no')

-- Math short cuts
local mathAbs = math.abs
local HALF_SQUARE_ROOT = math.sqrt(2.0) / 2.0
local QUARTER_PI = math.pi / 4.0
local sh, sw = love.graphics.getHeight(), love.graphics.getWidth()
------------------

-- dpad related variables
DPAD = {
    radius = 96,
    center = {x = 200, y = 250},
    isDown = {},
    overshoot = 20
}
DPAD.center.x = DPAD.radius + 5
DPAD.center.y = sh - DPAD.radius - 5
DPAD.radiusSquared = DPAD.radius * DPAD.radius
DPAD.deadPointRadius = DPAD.radius/9
DPAD.deadPointRadiusSquared = DPAD.deadPointRadius*DPAD.deadPointRadius
local radianOvershoot = DPAD.overshoot*math.pi/180
-------------------------

-- button related variables
button = {
  size = 69,
  radius = 88
}

---------------------------

local function buttonPress(x, y, s, px, py)
  return px > x and px < x+s and py > y and py < y+s
end

function drawCenteredText(rectX, rectY, rectWidth, rectHeight, text, scale)
	local font       = love.graphics.getFont()
	local textWidth  = font:getWidth(text)
	local textHeight = font:getHeight()
	love.graphics.print(text, rectX+rectWidth/2, rectY+rectHeight/2, nil, scale, nil, textWidth/2, textHeight/2)
end

function getPosition(angleDegrees, length)
    local angleRadians = angleDegrees * math.pi / 180
    local x = length * math.cos(angleRadians)
    local y = length * math.sin(angleRadians)
    return x, y
end



function DPAD:update(dt)
  
  button.center = {x= sw - DPAD.radius , y= sh - DPAD.radius }
  local buttonHalfSize = button.size/2
  local buttonOffset = button.radius - button.size
  local touches = love.touch.getTouches()
  
  local directionState = {up = false,down = false,left = false,right = false,h = false, v = false}
  local buttonState = {y = false,a = false,x = false,b = false}
  
  local buttonPos = {
    y = {x= button.center.x - (buttonHalfSize), y= button.center.y - (button.radius)},
    a = {x= button.center.x - (buttonHalfSize), y= button.center.y + (buttonOffset)},
    x = {x= button.center.x - (button.radius) , y= button.center.y - (buttonHalfSize)},
    b = {x= button.center.x + (buttonOffset), y= button.center.y - (buttonHalfSize)}
  }
  
  for i,id in ipairs(touches) do
    
    local mx, my = love.touch.getPosition(id)
    
    local mouseVectorX = mx - DPAD.center.x
    local mouseVectorY = my - DPAD.center.y
    
    -- check if you are touching the dpad
    local mouseVectorLengthSquared = (mouseVectorX * mouseVectorX
                                     + mouseVectorY * mouseVectorY)
    
    local isInRange = false
    if mouseVectorLengthSquared < DPAD.radiusSquared and
      mouseVectorLengthSquared > DPAD.deadPointRadiusSquared then
      isInRange = true
    end
    
    -- check wich direction is touched
    local vecAngle = math.deg(math.atan2(mouseVectorX, mouseVectorY))*-1 + 90
    if isInRange then
      if (vecAngle > -90 and vecAngle < -45 + DPAD.overshoot) or (vecAngle > 225 - DPAD.overshoot and vecAngle < 270) then directionState.up = true
        directionState.v = true end
      if vecAngle > 45 - DPAD.overshoot and vecAngle < 135 + DPAD.overshoot then directionState.down = true 
        directionState.v = true end
      if vecAngle > 135 - DPAD.overshoot and vecAngle < 225 + DPAD.overshoot then directionState.left = true
        directionState.h = true end
      if vecAngle > -45 - DPAD.overshoot and vecAngle < 45 + DPAD.overshoot then directionState.right = true
        directionState.h = true end
    end
    
    --check wich button is touched
    if buttonPress(buttonPos.y.x, buttonPos.y.y, button.size, mx, my) then
      buttonState.y = true end
    if buttonPress(buttonPos.a.x, buttonPos.a.y, button.size, mx, my) then
      buttonState.a = true end
    if buttonPress(buttonPos.x.x, buttonPos.x.y, button.size, mx, my) then
      buttonState.x = true end
    if buttonPress(buttonPos.b.x, buttonPos.b.y, button.size, mx, my) then
      buttonState.b = true end
  end
  
  
  -- updates wich direction are being touched
  DPAD.isDown['up'] = directionState.up
  DPAD.isDown['down'] = directionState.down
  DPAD.isDown['left'] = directionState.left
  DPAD.isDown['right'] = directionState.right
  
  local hrisontal = directionState.h
  local vertical = directionState.v
  
  DPAD.diagonal = false
  if hrisontal and vertical then
    DPAD.diagonal = true
  end
  
  -- updates wich button is being touched
  DPAD.isDown['y'] = buttonState.y
  DPAD.isDown['a'] = buttonState.a
  DPAD.isDown['x'] = buttonState.x
  DPAD.isDown['b'] = buttonState.b
end

function DPAD:draw()
    local buttonHalfSize = button.size/2
    local buttonOffset = button.radius - button.size
    mx, my = love.mouse.getPosition()
    
    love.graphics.setColor(0.7,0.7,0.7, 0.3)--col2
    love.graphics.circle('fill', DPAD.center.x, DPAD.center.y, DPAD.radius, DPAD.radius)
    
    love.graphics.setLineWidth(1.0)
    love.graphics.setColor(0.5, 0.5, 0.5)--col3
    
    local flx, fly = getPosition(180+45-DPAD.overshoot, DPAD.radius)
    local lx, ly = getPosition(180+45+DPAD.overshoot, DPAD.radius)
    local rx, ry = getPosition(-45-DPAD.overshoot, DPAD.radius)
    local frx, fry = getPosition(-45+DPAD.overshoot, DPAD.radius)
    
    love.graphics.line(DPAD.center.x+flx, DPAD.center.y+fly, DPAD.center.x-flx, DPAD.center.y-fly)
    love.graphics.line(DPAD.center.x+lx, DPAD.center.y+ly, DPAD.center.x-lx, DPAD.center.y-ly)
    love.graphics.line(DPAD.center.x+rx, DPAD.center.y+ry, DPAD.center.x-rx, DPAD.center.y-ry)
    love.graphics.line(DPAD.center.x+frx, DPAD.center.y+fry, DPAD.center.x-frx, DPAD.center.y-fry)
    
    --love.graphics.setColor(1.0, 1.0, 0.0)
    --love.graphics.line(DPAD.center.x, DPAD.center.y, mx, my)
    
    -- debug info
    --[[local mouseVectorX = mx - DPAD.center.x
    local mouseVectorY = my - DPAD.center.y
    vecAngle = math.deg(math.atan2(mouseVectorX, mouseVectorY))*-1 + 90
    love.graphics.setColor(1.0, 1.0, 1.0)
    love.graphics.print('Mouse: ' .. mx .. ', ' .. my, 10, 0)
    love.graphics.print('angle: ' ..vecAngle, 10, 20)]]--
    
    
    if DPAD.isDown then
        love.graphics.setColor(0.9,0.9,0.9, 0.5)--col1
        
        local angle1, angle2 = 0,0
        if DPAD.isDown['up'] then
            angle1, angle2 = -QUARTER_PI+radianOvershoot, -QUARTER_PI * 3.0- radianOvershoot
            love.graphics.arc('fill', DPAD.center.x, DPAD.center.y, DPAD.radius, angle1, angle2)
        elseif DPAD.isDown['down'] then
            angle1, angle2 = QUARTER_PI-radianOvershoot, QUARTER_PI * 3.0+ radianOvershoot
            love.graphics.arc('fill', DPAD.center.x, DPAD.center.y, DPAD.radius, angle1, angle2)
        end
        if DPAD.isDown['right'] then
            angle1, angle2 = -QUARTER_PI-radianOvershoot, QUARTER_PI+radianOvershoot
            love.graphics.arc('fill', DPAD.center.x, DPAD.center.y, DPAD.radius, angle1, angle2)
        elseif DPAD.isDown['left'] then
            angle1, angle2 = QUARTER_PI * 3.0-radianOvershoot, QUARTER_PI * 5.0+ radianOvershoot
            love.graphics.arc('fill', DPAD.center.x, DPAD.center.y, DPAD.radius, angle1, angle2)
        end
    end
    love.graphics.setColor(0.7,0.7,0.7, 1.0)--col2
    love.graphics.setLineWidth(3.0)
    love.graphics.circle('line', DPAD.center.x, DPAD.center.y, DPAD.radius, DPAD.radius)
    
    love.graphics.circle('line', DPAD.center.x, DPAD.center.y, DPAD.deadPointRadius, DPAD.deadPointRadius)
  
  --back
  love.graphics.setColor(0.7,0.7,0.7, 0.3)--col2
  collorSwitch('y')
  love.graphics.rectangle('fill', button.center.x - (buttonHalfSize), button.center.y - (button.radius), button.size, button.size)
  collorSwitch('a')
  love.graphics.rectangle('fill', button.center.x - (buttonHalfSize), button.center.y + (buttonOffset), button.size, button.size)
  collorSwitch('x')
  love.graphics.rectangle('fill', button.center.x - (button.radius) , button.center.y - (buttonHalfSize), button.size, button.size)
  collorSwitch('b')
  love.graphics.rectangle('fill', button.center.x + (buttonOffset), button.center.y - (buttonHalfSize), button.size, button.size)
  
  --outline
  love.graphics.setColor(0.7,0.7,0.7, 1.0)--col2
  love.graphics.setLineWidth(2.0)
  love.graphics.rectangle('line', button.center.x - (buttonHalfSize), button.center.y - (button.radius), button.size, button.size)
  love.graphics.rectangle('line', button.center.x - (buttonHalfSize), button.center.y + (buttonOffset), button.size, button.size)
  love.graphics.rectangle('line', button.center.x - (button.radius) , button.center.y - (buttonHalfSize), button.size, button.size)
  love.graphics.rectangle('line', button.center.x + (buttonOffset), button.center.y - (buttonHalfSize), button.size, button.size)
  
  
  --text
  love.graphics.setColor(1, 1, 1,1)
  drawCenteredText(button.center.x, button.center.y- button.radius+ buttonHalfSize, 1, 1, 'y', 2)
  drawCenteredText(button.center.x, button.center.y+ button.radius- buttonHalfSize, 1, 1, 'a', 2)
  drawCenteredText(button.center.x- button.radius+ buttonHalfSize, button.center.y, 1, 1, 'x', 2)
  drawCenteredText(button.center.x+ button.radius- buttonHalfSize, button.center.y, 1, 1, 'b', 2)
  
  love.graphics.reset( )
end

function collorSwitch(b)
  if DPAD.isDown[b] then
    return love.graphics.setColor(0.9,0.9,0.9, 0.5)--col1
  end
  return love.graphics.setColor(0.7,0.7,0.7, 0.3)--col2
end